import csv
import psycopg2

from database import db, app
from flask_covid19.blueprints.app_all.all_config import BlueprintConfig
from flask_covid19.blueprints.app_web.web_model_factory import BlueprintDateReportedFactory
from flask_covid19.blueprints.data_rki.rki_model import RkiMeldedatum
from flask_covid19.blueprints.data_rki.rki_model_import import RkiImport, RkiFlat


class RkiServiceImport:
    def __init__(self, database, config: BlueprintConfig):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RKI Service Import [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = config
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RKI Service Import [ready]")

    def import_file(self):
        app.logger.info(" RkiServiceImport.import_file()  [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info(" import into TABLE: "+self.cfg.tablename+" <--- from FILE "+self.cfg.cvsfile_path)
        app.logger.info("------------------------------------------------------------")
        row = None
        try:
            RkiImport.remove_all()
            RkiFlat.remove_all()
            with open(self.cfg.cvsfile_path, newline='\n') as csv_file:
                file_reader = csv.DictReader(csv_file, delimiter=',', quotechar='"')
                k = 0
                for row in file_reader:
                    k = k + 1
                    meldedatum = row['Meldedatum']
                    d = BlueprintDateReportedFactory.create_new_object_for_rki(my_date_rep=meldedatum)
                    # app.logger.info("RKI meldedatum: " + str(d) + " " + meldedatum)
                    o = RkiImport(
                        date_reported_import_str=meldedatum,
                        fid=row['FID'],
                        id_bundesland=row['IdBundesland'],
                        bundesland=row['Bundesland'],
                        landkreis=row['Landkreis'],
                        altersgruppe=row['Altersgruppe'],
                        geschlecht=row['Geschlecht'],
                        anzahl_fall=row['AnzahlFall'],
                        anzahl_todesfall=row['AnzahlTodesfall'],
                        meldedatum=row['Meldedatum'],
                        id_landkreis=row['IdLandkreis'],
                        datenstand=row['Datenstand'],
                        neuer_fall=row['NeuerFall'],
                        neuer_todesfall=row['NeuerTodesfall'],
                        ref_datum=row['Refdatum'],
                        neu_genesen=row['NeuGenesen'],
                        anzahl_genesen=row['AnzahlGenesen'],
                        ist_erkrankungsbeginn=row['IstErkrankungsbeginn'],
                        altersgruppe2=row['Altersgruppe2'],
                        datum=d.datum,
                        processed_update=False,
                        processed_full_update=False,
                    )
                    db.session.add(o)
                    oo = RkiFlat(
                        datum=d.datum,
                        year=d.year,
                        month=d.month,
                        day_of_month=d.day_of_month,
                        day_of_week=d.day_of_week,
                        week_of_year=d.week_of_year,
                        day_of_year=d.day_of_year,
                        year_week=d.year_week,
                        year_day_of_year=d.year_day_of_year,
                        date_reported_import_str=meldedatum,
                        year_month=d.year_month,
                        location_code=row['IdLandkreis'],
                        location=row['Landkreis'],
                        location_group=row['Bundesland'],
                        processed_update=False,
                        processed_full_update=False,
                        #
                        fid=row['FID'],
                        id_bundesland=row['IdBundesland'],
                        bundesland=row['Bundesland'],
                        landkreis=row['Landkreis'],
                        altersgruppe=row['Altersgruppe'],
                        geschlecht=row['Geschlecht'],
                        anzahl_fall=row['AnzahlFall'],
                        anzahl_todesfall=row['AnzahlTodesfall'],
                        meldedatum=row['Meldedatum'],
                        id_landkreis=row['IdLandkreis'],
                        datenstand=row['Datenstand'],
                        neuer_fall=row['NeuerFall'],
                        neuer_todesfall=row['NeuerTodesfall'],
                        ref_datum=row['Refdatum'],
                        neu_genesen=row['NeuGenesen'],
                        anzahl_genesen=row['AnzahlGenesen'],
                        ist_erkrankungsbeginn=row['IstErkrankungsbeginn'],
                        altersgruppe2=row['Altersgruppe2'],
                    )
                    db.session.add(oo)
                    if (k % 2000) == 0:
                        db.session.commit()
                        app.logger.info(" import RKI ... " + str(k) + " rows")
                    if k == 230091:
                        break
                db.session.commit()
                app.logger.info(" import RKI ... " + str(k) + " rows total")
        except KeyError as error:
            app.logger.warning("WARN: import RKI [begin]")
            app.logger.warning(":::"+str(error)+":::")
            for item_key, item_value in row.items():
                app.logger.warning(str(item_key)+" : "+str(item_value))
            app.logger.warning("WARN:import RKI [end]")
        except (Exception, psycopg2.DatabaseError) as error:
            app.logger.warning("WARN:import RKI [begin]")
            app.logger.warning(error)
            app.logger.warning("WARN:import RKI [end]")
        finally:
            app.logger.info("")
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" imported into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
            app.logger.info("------------------------------------------------------------")
            app.logger.info("RkiServiceImport.import_file()  [done]")
        return self
